#include "slog_preview.h"
#include "bswp_fileio.h"



int SLOG_WritePreview( SLOG_STREAM *slog )
{
    const char        str[] = "|SLOG---PREVIEW|";
          SLOG_uint32 Nchar;
          int         ierr;

    /*  Update the content of SLOG_hdr_t to both memory and disk  */
    slog->hdr->fptr2preview = ( SLOG_fptr ) ftell( slog->fd );
    ierr = SLOG_WriteUpdatedHeader( slog );
    if ( ierr != SLOG_SUCCESS ) {
        fprintf( errfile,  __FILE__":SLOG_WritePreview() - "
                                   "SLOG_WriteUpdatedHeader() fails\n" );
        fflush( errfile );
        return ierr;
    }

    /*  Write the content of SLOG Preview to the disk  */
    Nchar = strlen( str );

    ierr = bswp_fwrite( &Nchar, SLOG_typesz[ ui32 ], 1, slog->fd );
    if ( ierr != 1 ) {
        fprintf( errfile,  __FILE__":SLOG_WritePreview() - "
                           "Writing number of characters fails\n" );
        fflush( errfile );
        return SLOG_EOF;
    }

    ierr = bswp_fwrite( str, 1, Nchar, slog->fd );
    if ( ierr != Nchar ) {
        fprintf( errfile,  __FILE__":SLOG_WritePreview() - "
                           "Writing string fails\n" );
        fflush( errfile );
        return SLOG_EOF;
    }

    return SLOG_SUCCESS;
}



int SLOG_ReadPreview( SLOG_STREAM *slog )
{
    char        *str;
    SLOG_uint32  Nchar;
    int          ierr;

    /*  Seek to the beginning of the Preview in the SLOG file  */
    if ( slog->hdr->fptr2preview == SLOG_fptr_NULL ) {
        fprintf( errfile, __FILE__":SLOG_ReadPreview() - "
                          "slog->hdr->fptr2preview == SLOG_fptr_NULL\n" );
        fprintf( errfile, "\t""SLOG_WritePreview() may NOT be called\n" );
        fflush( errfile );
        return SLOG_FAIL;
    }
    ierr = fseek( slog->fd, slog->hdr->fptr2preview, SEEK_SET );
    if ( ierr != 0 ) {
        fprintf( errfile, __FILE__":SLOG_ReadPreview() - "
                          "fseek( fptr2preview ) fails\n" );
        fflush( errfile );
        return SLOG_FAIL;
    }

    /*  Read in the Preview  */
    ierr = bswp_fread( &Nchar, SLOG_typesz[ ui32 ], 1, slog->fd );
    if ( ierr != 1 ) {
        fprintf( errfile,  __FILE__":SLOG_ReadPreview() - "
                           "Reading number of characters fails\n" );
        fflush( errfile );
        return SLOG_EOF;
    }

    str = ( char * ) malloc( (Nchar+1) * sizeof( char ) );
    if ( str == NULL ) {
        fprintf( errfile,  __FILE__":SLOG_ReadPreview() - "
                          "malloc() fails\n" );
        fflush( errfile );
        return SLOG_FAIL;
    }

    ierr = bswp_fread( str, 1, Nchar, slog->fd );
    if ( ierr != Nchar ) {
        fprintf( errfile,  __FILE__":SLOG_ReadPreview() - "
                           "Reading string fails\n" );
        fflush( errfile );
        return SLOG_EOF;
    }

    str[ Nchar ] = '\0';

#if defined( DEBUG )
    fprintf( outfile, __FILE__":SLOG_ReadPreview() - Read :\n" );
    fprintf( outfile, "       ""%s\n", str );
    fflush( outfile );
#endif

    return SLOG_SUCCESS;
}
